local f = CreateFrame("Frame", nil, UIParent);
local button, barExp, barRep, barExpText, barRepText;


function TitanXPBar_GetButtonText()
  if (barExp:IsVisible() or barRep:IsVisible()) then
    return "";
  else
    return "TitanXPBar: Nothing to show!";
  end
end

function TitanXPBar_GetTooltipText()
  local tipText = "";

  if UnitLevel("player") ~= MAX_PLAYER_LEVEL then
    local restedXP    = GetXPExhaustion();
    local pXP, pMaxXP = UnitXP("player"), UnitXPMax("player");
    
    -- exp and required exp for the next level
    tipText = tipText .. XP .. ":\t" .. pXP .. "/" .. pMaxXP;
    
    -- rested exp, if any
    if restedXP then
      tipText = tipText .. "\n" .. format("Rested for another %d XP.", restedXP);
    end
  end
  
  -- watched faction, if any
  local fStandingText, rBarMin, rBarMax, rBarVal;
  local fName, fStanding, rMin, rMax, rCur = GetWatchedFactionInfo();
  
  local fID, isWatched, tFactionID = 0;
  for i = 1, GetNumFactions() do
    isWatched, _, tFactionID = select(12, GetFactionInfo(i));
    if isWatched == 1 then
      fID = tFactionID;
      break;
    end
  end
  
  local frID, frRep, _, _, _, _, frTextLevel, frThreshold, frNextThreshold = GetFriendshipReputation(fID);
  
  -- see if the watched faction is a friend faction
  if frID ~= nil then
    fStandingText = frTextLevel;
    
    if frNextThreshold then
      rBarMin, rBarMax, rBarVal = frThreshold, frNextThreshold, frRep;
    else
      rBarMin, rBarMax, rBarVal = 0, 1, 1;
    end
  else
    fStandingText = _G["FACTION_STANDING_LABEL" .. fStanding];
    rBarMin, rBarMax, rBarVal = 0, rMax - rMin, rCur - rMin;
  end
  
  if fName then
    local col = FACTION_BAR_COLORS[fStanding];
    tipText = tipText .. "\n\n" .. format("|cff%02x%02x%02x%s|r (%s):\t%d/%d", col.r * 255, col.g * 255, col.b * 255, fName, fStandingText, rBarVal, rBarMax);
  end
  
  return tipText;
end


local function UpdateBars()
  local fName, fStanding, rMin, rMax, rCur = GetWatchedFactionInfo();
  
  -- we are watching a faction, apparently
  if fName then
    local fID, isWatched, tFactionID = 0;
    for i = 1, GetNumFactions() do
      isWatched, _, tFactionID = select(12, GetFactionInfo(i));
      if isWatched == 1 then
        fID = tFactionID;
        break;
      end
    end
    
    local rBarMin, rBarMax, rBarVal;
    local frID, frRep, _, _, _, _, frTextLevel, frThreshold, frNextThreshold = GetFriendshipReputation(fID);
    
    -- see if the watched faction is a friend faction
    if frID ~= nil then
      fStanding = 5; -- friend factions are always green
      if frNextThreshold then
        rBarMin, rBarMax, rBarVal = frThreshold, frNextThreshold, frRep;
      else
        rBarMin, rBarMax, rBarVal = 0, 1, 1;
      end
    else
      rBarMin, rBarMax, rBarVal = 0, rMax - rMin, rCur - rMin;
    end
    
    barRep:Show();
    
    local color = FACTION_BAR_COLORS[fStanding];
    local rPos  = rBarVal / rBarMax * 100;
    
    barRep:SetStatusBarColor(color.r, color.g, color.b);
    barRep:SetMinMaxValues(0, rBarMax);
    barRep:SetValue(rBarVal);
    barRepText:SetText(format("%d%%", rPos));
  else
    barRep:Hide();
  end
  
  if UnitLevel("player") == MAX_PLAYER_LEVEL then
    barExp:Hide();
    
    -- fix rep bar position
    barRep:ClearAllPoints();
    barRep:SetPoint("LEFT", button, "LEFT");
  else
    barExp:Show();
    
    -- fix rep bar position
    barRep:ClearAllPoints();
    barRep:SetPoint("LEFT", barExp, "RIGHT", 8, 0);
  end
  
  local restState   = GetRestState();
  local pXP, pMaxXP = UnitXP("player"), UnitXPMax("player");
  local rPos = (pXP / pMaxXP) * 100;
  
  if restState == 1 then
    barExp:SetStatusBarColor(0, 0.39, 0.88);
  else
    barExp:SetStatusBarColor(0.58, 0, 0.55);
  end
  
  barExp:SetMinMaxValues(0, pMaxXP);
  barExp:SetValue(pXP);
  if rPos >= 0 then
    barExpText:SetText(format("%d%%", rPos));
  end
end

local TitanPanelButton_UpdateButton_old = TitanPanelButton_UpdateButton;
local function UpdateButtonWidth(id, setButtonWidth)
  local w = 0;
  if (id == "XPBar") then
    if barExp:IsVisible() then
      w = w + 100;
    end
    
    if barRep:IsVisible() then
      if w > 0 then
        w = w + 108;
      else
        w = w + 100;
      end
    end
  end
  
  if (barExp and barRep and w > 0) then
    local newWidth = w;
    TitanPanelButton_UpdateButton_old(id, setButtonWidth);
    return TitanPanelXPBarButton:SetWidth(newWidth);
  else
    return TitanPanelButton_UpdateButton_old(id, setButtonWidth);
  end
end

local function Init()
  button = CreateFrame("Button", "TitanPanelXPBarButton", f, "TitanPanelTextTemplate");
  button:SetFrameStrata("FULLSCREEN");
  
  button.registry = {
    id           = "XPBar",
    category     = "Information",
    version      = "1.0",
    menuText     = "XPBar",
    tooltipTitle = XP .. " / " .. REPUTATION,
    buttonTextFunction  = "TitanXPBar_GetButtonText",
    tooltipTextFunction = "TitanXPBar_GetTooltipText",
    
    controlVariables = {
      ShowIcon           = false,
      ShowLabelText      = true,
      ShowRegularText    = false,
      ShowColoredText    = false,
      DisplayOnRightSide = false
    }
  };
  
  -- basic event handlers
  button:SetScript("OnShow",  TitanPanelButton_OnShow);
  button:SetScript("OnClick", TitanPanelButton_OnClick);
  
  -- oh, and unregister the event
  f:UnregisterEvent("ADDON_LOADED");
  
  
  -- alright, here comes the actual magic. we will swiftly avoid Titan Panel's limits and implement some awesome shit~
  barExp = CreateFrame("StatusBar", "TitanPanelXPBarOne", button);
  
  barExp:SetStatusBarTexture([[Interface\TargetingFrame\UI-StatusBar]]);
  barExp:GetStatusBarTexture():SetHorizTile(false);
	barExp:GetStatusBarTexture():SetVertTile(false);
  barExp:SetBackdrop({ 
		bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
		insets = { left = 0, right = 0, top = 0, bottom = 0},
  });
  barExp:SetBackdropColor(0, 0, 0, 0.5);
  
  barExp:SetWidth(100);
  barExp:SetHeight(10);
  barExp:SetPoint("LEFT", button, "LEFT");
  
  
  barExpText = barExp:CreateFontString(nil, "OVERLAY");
  barExpText:SetFont("Fonts\\FRIZQT__.TTF", 8);
  barExpText:SetShadowOffset(1, -1);
  barExpText:SetShadowColor(0, 0, 0, 1);
  barExpText:SetWidth(100);
  barExpText:SetPoint("CENTER", barExp, "CENTER");
  
  
  
  barRep = CreateFrame("StatusBar", "TitanPanelXPBarTwo", button);
  
  barRep:SetStatusBarTexture([[Interface\TargetingFrame\UI-StatusBar]]);
  barRep:GetStatusBarTexture():SetHorizTile(false);
	barRep:GetStatusBarTexture():SetVertTile(false);
  barRep:SetBackdrop({ 
		bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
		insets = { left = 0, right = 0, top = 0, bottom = 0},
  });
  barRep:SetBackdropColor(0, 0, 0, 0.5);
  
  barRep:SetWidth(100);
  barRep:SetHeight(10);
  barRep:SetPoint("LEFT", barExp, "RIGHT", 8, 0);
  
  
  barRepText = barRep:CreateFontString(nil, "OVERLAY");
  barRepText:SetFont("Fonts\\FRIZQT__.TTF", 8);
  barRepText:SetShadowOffset(1, -1);
  barRepText:SetShadowColor(0, 0, 0, 1);
  barRepText:SetWidth(100);
  barRepText:SetPoint("CENTER", barRep, "CENTER");
  
  
  -- we're the recincarnation of all evil, so let's hook this function to ensure correct width for our "button"
  TitanPanelButton_UpdateButton = UpdateButtonWidth;
end


f:RegisterEvent("ADDON_LOADED");
f:RegisterEvent("PLAYER_XP_UPDATE");
f:RegisterEvent("PLAYER_UPDATE_RESTING");
f:RegisterEvent("PLAYER_LEVEL_UP");
f:RegisterEvent("UPDATE_FACTION");
f:RegisterEvent("UPDATE_EXHAUSTION");

f:SetScript("OnEvent", function(self, event, addon)
  if (event == "ADDON_LOADED" and addon == "TitanXPBar") then
    Init();
    return;
  end
  
  if not barExp then
    return;
  end
  
  UpdateBars();
  
  if event == "UPDATE_FACTION" then
    UpdateButtonWidth("XPBar", 1);
  end
end);